//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2005 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import com.aspose.words.Document;
import com.aspose.words.License;
import com.aspose.words.SaveFormat;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;

//Java: In a whole, this class corresponds to C# Aspose.Words.Demos.Viewer.aspx.cs.

public class DemoServlet extends HttpServlet
{
    // Commonly launched only once, when an application server initializes the servlet.
    public void init() throws ServletException
    {
		// Get the context path of ours web application.
        mWebAppDir = this.getServletContext().getRealPath("/");

		//load the license and init the demo
		//rethrow added in order to avoid changing this method's signature.
		try
		{
			Demo.initDemo();

			mGeneratedDir = new File(mWebAppDir, GENERATED_DIR_SHORT_NAME);
			if (!mGeneratedDir.exists())
				mGeneratedDir.mkdir();

			// *** Licensing: Put the license file into the /WEB-INF folder,
			// otherwise Aspose.Words will work in evaluation mode.
			loadLicense();
		}
		catch (Exception e)
		{
			throw new ServletException(e);
		}
    }

	private void loadLicense() throws Exception
	{
		//servlet container prohibits an external access to the WEB-INF folder.
		File licenseFile = new File(mWebAppDir, "WEB-INF/Aspose.Words.Java.lic");
		if (licenseFile.exists())
		{
			//This shows how to license Aspose.Words.
			//If you don't specify a license, Aspose.Words works in evaluation mode.
			License license = new License();
			license.setLicense(licenseFile.getAbsolutePath());
		}
	}

    /**
     * Creates and returns an object that can run the demo selected by the user.
     */
    private Demo createDemo(String demoName)
    {
		if ("AlternatingRows".equals(demoName))
			return new AlternatingRowsDemo();
		else if ("DinnerInvitation".equals(demoName))
			return new DinnerInvitationDemo();
		else if ("DocumentBuilder".equals(demoName))
			return new DocumentBuilderDemo();
		else if ("EmployeesReport".equals(demoName))
			return new EmployeesReportDemo();
		else if ("MailingLabels".equals(demoName))
			return new MailingLabelsDemo();
		else if ("ProductCatalog".equals(demoName))
			return new ProductCatalogDemo();
		else if ("SalesInvoice".equals(demoName))
			return new SalesInvoiceDemo();
		else if ("StampAndWatermark".equals(demoName))
			return new StampAndWatermarkDemo();
		else
			throw new RuntimeException("Unknown demo selected.");
    }

    // Called from the web-app index page (because i choose POST method for input form).
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        // Get the demo selected by the user.
        String demoName = request.getParameter("demo");

        // Get the output format selected by user.
        String formatType = request.getParameter("format");

        // Java-changed: if null, do default demo (in C# - do nothing).
        if (demoName == null)
            demoName = "DinnerInvitation";

        // Create and execute the demo, it produces a document in memory.
        Demo demo = createDemo(demoName);

        // Java-added rethrow in order to avoid changing this method's signature.
        try
        {
            Document doc = demo.execute();

            // Once we have a document, we can save it to a file, stream or send to the client browser.
            // We just send the document to the browser in this case.
            boolean openNewWindow = request.getParameter("openNewWindow") != null;
            sendToBrowser(doc, demoName, formatType, openNewWindow, response);
            response.flushBuffer();
        }
        catch (Exception e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * Sends the document to the client browser.
     */
    private void sendToBrowser(Document doc, String demoName, String formatType, boolean openNewWindow, HttpServletResponse response)
            throws Exception
    {
        String fileName = demoName + "." + formatType;

		//Add Response header
        if(openNewWindow)
            response.setHeader("content-disposition","attachment; filename=" + fileName);
        else
            response.addHeader("content-disposition","inline; filename=" + fileName);

        if("DOC".equals(formatType))
        {
            //Set Response content type
            response.setContentType("application/msword");

            //Finally save generated document to Response Out.
            doc.save(response.getOutputStream(), SaveFormat.DOC);
        }
		else if("DOCX".equals(formatType))
		{
			response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			doc.save(response.getOutputStream(), SaveFormat.DOCX);
		}
        else if("HTML".equals(formatType))
        {
			sendToBrowserAsHtml(doc, response, demoName);
		}
        else if("TXT".equals(formatType))
        {
            response.setContentType("text/plain");
            doc.save(response.getOutputStream(), SaveFormat.TEXT);
        }
		else if("PDF".equals(formatType))
		{
			response.setContentType("application/pdf");
			Doc2PdfConverter d2p = new Doc2PdfConverter(response.getOutputStream());
			doc.accept(d2p);
		}
    }

	/**
	 * Stream the document to the client browser in HTML format.
	 *
	 * Option 1: Save the document into a file and redirect this file
	 * (along with image files saved in the same folder) to the client browser.
	 */
	private void sendToBrowserAsHtml(Document doc, HttpServletResponse response, String fileName) throws Exception
	{
		//Output the XHTML header.
		doc.getSaveOptions().setHtmlExportXhtmlTransitional(true);

		//save the document and it's images into "generated" subdir uder web app dir.
		String relativeUrl = GENERATED_DIR_SHORT_NAME + "/" + fileName;
		String realPath = this.getServletContext().getRealPath(relativeUrl);
		doc.save(realPath, SaveFormat.HTML);
		//Redirecting to the HTML document rather than saving it into the response stream
		//makes sure the browser can find the images in the same folder as the document.
		response.setContentType("text/html");
		response.sendRedirect(relativeUrl);
	}

	/**
	 * Stream the document to the client browser in HTML format.
	 *
	 * Option 2: Save the document into the response stream.
	 * Illustrates how to use ExportImagesFolder and HtmlExportImagesFolderAlias properties.
	 *
	 * Note that aspose.words generates unique name for each image file (when saving the doc
	 * in html format to a stream) so you need in some logic to wipe-out these generated files
	 * or save them into some kind of a temp dir.
	 */
	private void sendToBrowserAsHtmlResponse(Document doc, HttpServletResponse response) throws Exception
	{
		response.setContentType("text/html");
		//Output the XHTML header.
		doc.getSaveOptions().setHtmlExportXhtmlTransitional(true);
		//Image paths for the server (local) - aspose.words puts generated image files here.
		doc.getSaveOptions().setExportImagesFolder(mGeneratedDir.getPath());
		//Image paths for the browser (external) - aspose.words calculates external
		//links to the generated images and puts them inside generated html page.
		doc.getSaveOptions().setHtmlExportImagesFolderAlias(GENERATED_DIR_SHORT_NAME);
		doc.save(response.getOutputStream(), SaveFormat.HTML);
	}

	private String mWebAppDir;
    private File mGeneratedDir;
    private static final String GENERATED_DIR_SHORT_NAME = "generated";
}

