//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

package com.aspose.words.demos;

import com.aspose.words.Document;
import com.aspose.words.License;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;

/**
 * Allows selecting and launching demos by GUI form.
 */
public class DemoGUI extends JFrame
{
	public static void main(String[] args) throws Exception
	{
		//init the demo
		Demo.initDemo();
		loadLicense();

		//launch the GUI
		DemoGUI frame = new DemoGUI();
		frame.setVisible(true);
	}

	private static void loadLicense() throws Exception
	{
		File licenseFile = new File(Demo.getRootDistributiveDir(), "Aspose.Words.Java.lic");
		if (licenseFile.exists())
		{
			//This shows how to license Aspose.Words.
			//If you don't specify a license, Aspose.Words works in evaluation mode.
			License license = new License();
			license.setLicense(licenseFile.getAbsolutePath());
		}
	}

	/**
	 * Returns the currently selected demo name.
	 */
	private String getDemoName()
	{
		return demoGroup.getSelection().getActionCommand();
	}

	/**
	 * Returns the currently selected format.
	 */
	private String getSelectedFormat()
	{
		return formatGroup.getSelection().getActionCommand();
	}

	/**
	 * Geths the output file name that consists of the selected demo name and selected output format.
	 */
	private String getProposedFileName()
	{
		return getDemoName() + "." + getSelectedFormat();
	}

	private void onSave()
	{
		try
		{
			//Show the save dialog to select the destination file name and then run the demo.
			JFileChooser saveDialog = new JFileChooser();
			saveDialog.setSelectedFile(new File(getProposedFileName()));

			String outFileName;
			if (saveDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				outFileName = saveDialog.getSelectedFile().getAbsolutePath();
				String fileNameWithoutExt = outFileName.substring(0, outFileName.lastIndexOf("."));
				runDemo(fileNameWithoutExt);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, e, this.getTitle(), JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Runs the currently selected demo and saves the document into the specified file name.
	 */
	private void runDemo(String dstFileName) throws Exception
	{
		Demo demo = createDemo(getDemoName());

		//Execute the demo to generate the report.
		Document doc = demo.execute();

		// Convert from a DOC to a PDF, if user choose PDF format as output format
		if ("pdf".equals(getSelectedFormat()))
		{
			// Create an object that inherits from the DocumentVisitor class.
			Doc2PdfConverter d2p = new Doc2PdfConverter(new FileOutputStream(dstFileName + ".pdf"));
			doc.accept(d2p);
		}
		else
		{
			//Save the resulting document in choosed format (except of pdf).
			doc.save(dstFileName + "." + getSelectedFormat());
		}
	}

	/**
	 * Creates and returns an object that can run the demo selected by the user.
	 */
	private Demo createDemo(String demoName) throws Exception
	{
		if ("Alternating Rows".equals(demoName))
			return new AlternatingRowsDemo();
		else if ("Dinner Invitation".equals(demoName))
			return new DinnerInvitationDemo();
		else if ("Document Builder".equals(demoName))
			return new DocumentBuilderDemo();
		else if ("Employees Report".equals(demoName))
			return new EmployeesReportDemo();
		else if ("Mailing Labels".equals(demoName))
			return new MailingLabelsDemo();
		else if ("Product Catalog".equals(demoName))
			return new ProductCatalogDemo();
		else if ("Sales Invoice".equals(demoName))
			return new SalesInvoiceDemo();
		else if ("Stamp and Watermark".equals(demoName))
			return new StampAndWatermarkDemo();
		else
			throw new Exception("Unknown demo selected.");
	}

	//============== GUI staff ===============

	public DemoGUI()
	{
		initComponents();
		getRootPane().setDefaultButton(saveButton);
	}

	private void saveButtonActionPerformed(ActionEvent e)
	{
		onSave();
	}

	private void closeButtonActionPerformed(ActionEvent e)
	{
		dispose();
	}

	private void initComponents()
	{
		preface = new JPanel();
		label1 = new JLabel();
		textPane1 = new JTextPane();
		textPane2 = new JTextPane();
		textPane3 = new JTextPane();
		contentPanel = new JPanel();
		demoGroupBox = new JPanel();
		dinnerInvitationRadio = new JRadioButton();
		dinnerInvitationText = new JTextPane();
		mailingLabelsRadio = new JRadioButton();
		mailingLabelsText = new JTextPane();
		employeesReportRadio = new JRadioButton();
		employeesReportText = new JTextPane();
		salesInvoiceRadio = new JRadioButton();
		salesInvoiceText = new JTextPane();
		productCatalogRadio = new JRadioButton();
		productCatalogText = new JTextPane();
		alternatingRowsRadio = new JRadioButton();
		alternatingRowsText = new JTextPane();
		documentBuilderRadio = new JRadioButton();
		documentBuilderText = new JTextPane();
		stampAndWatermarkRadio = new JRadioButton();
		stampAndWatermarkText = new JTextPane();
		formatGroupBox = new JPanel();
		docButton = new JRadioButton();
		docxButton = new JRadioButton();
		htmlButton = new JRadioButton();
		txtButton = new JRadioButton();
		pdfButton = new JRadioButton();
		buttonBar = new JPanel();
		saveButton = new JButton();
		closeButton = new JButton();
		demoGroup = new ButtonGroup();
		formatGroup = new ButtonGroup();

		//======== this ========
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		setTitle("Aspose.Words.Demos");
		setVisible(true);
		setResizable(false);
		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());

		//======== preface ========
		{
			preface.setFont(new Font("Tahoma", Font.PLAIN, 12));
			preface.setBorder(new EmptyBorder(5, 5, 5, 5));
			preface.setFocusable(false);
			preface.setPreferredSize(new Dimension(657, 141));
			preface.setMinimumSize(new Dimension(657, 141));
			preface.setMaximumSize(new Dimension(657, 141));
			preface.setLayout(new BoxLayout(preface, BoxLayout.PAGE_AXIS));

			//---- label1 ----
			label1.setText("Welcome to the Aspose.Words Featured Demos!");
			label1.setFont(new Font("Tahoma", Font.BOLD, 14));
			label1.setIconTextGap(0);
			label1.setFocusable(false);
			label1.setOpaque(true);
			preface.add(label1);

			//---- textPane1 ----
			textPane1.setFont(new Font("Tahoma", Font.PLAIN, 12));
			textPane1.setText("Important: To produce Word documents, the machine to run Aspose.Words does not need to have Microsoft Word installed as Aspose.Words itself is a Word document creation engine.");
			textPane1.setOpaque(false);
			textPane1.setEditable(false);
			textPane1.setFocusable(false);
			textPane1.setMargin(new Insets(9, 3, 3, 3));
			preface.add(textPane1);

			//---- textPane2 ----
			textPane2.setFont(new Font("Tahoma", Font.PLAIN, 12));
			textPane2.setText("In all of these examples no OLE automation is used, all work is performed by Aspose.Words without Microsoft Word installed.");
			textPane2.setOpaque(false);
			textPane2.setEditable(false);
			textPane2.setFocusable(false);
			preface.add(textPane2);

			//---- textPane3 ----
			textPane3.setFont(new Font("Tahoma", Font.PLAIN, 12));
			textPane3.setText("Garbage text and evaluation watermark are injected into documents when this demo runs without a license in evaluation mode.");
			textPane3.setOpaque(false);
			textPane3.setForeground(Color.red);
			textPane3.setEditable(false);
			textPane3.setFocusable(false);
			preface.add(textPane3);
		}
		contentPane.add(preface, BorderLayout.NORTH);

		//======== contentPanel ========
		{
			contentPanel.setFont(new Font("Tahoma", Font.PLAIN, 12));
			contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
			contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.PAGE_AXIS));

			//======== demoGroupBox ========
			{
				demoGroupBox.setBorder(new TitledBorder("Demo type"));
				demoGroupBox.setFont(new Font("Tahoma", Font.PLAIN, 12));
				demoGroupBox.setLayout(new GridBagLayout());
				((GridBagLayout)demoGroupBox.getLayout()).columnWidths = new int[] {0, 0, 0};
				((GridBagLayout)demoGroupBox.getLayout()).rowHeights = new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0};
				((GridBagLayout)demoGroupBox.getLayout()).columnWeights = new double[] {0.0, 0.0, 1.0E-4};
				((GridBagLayout)demoGroupBox.getLayout()).rowWeights = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0E-4};

				//---- dinnerInvitationRadio ----
				dinnerInvitationRadio.setText("Dinner Invitation");
				dinnerInvitationRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				dinnerInvitationRadio.setSelected(true);
				dinnerInvitationRadio.setActionCommand("Dinner Invitation");
				demoGroupBox.add(dinnerInvitationRadio, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
																			   GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			   new Insets(0, 0, 5, 5), 0, 0));

				//---- dinnerInvitationText ----
				dinnerInvitationText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				dinnerInvitationText.setEditable(false);
				dinnerInvitationText.setOpaque(false);
				dinnerInvitationText.setText("Creates a letter for a list of clients defined in the database. Shows how to use \"simple\" mail merge, similar to the classic mail merge available in Microsoft Word.");
				dinnerInvitationText.setFocusable(false);
				demoGroupBox.add(dinnerInvitationText, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
																			  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			  new Insets(0, 0, 5, 0), 0, 0));

				//---- mailingLabelsRadio ----
				mailingLabelsRadio.setText("Mailing Labels");
				mailingLabelsRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				mailingLabelsRadio.setActionCommand("Mailing Labels");
				demoGroupBox.add(mailingLabelsRadio, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
																			GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			new Insets(0, 0, 5, 5), 0, 0));

				//---- mailingLabelsText ----
				mailingLabelsText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				mailingLabelsText.setEditable(false);
				mailingLabelsText.setOpaque(false);
				mailingLabelsText.setText("Prints company names and addresses on mailing labels. Shows how to use NEXT field and how to merge from DataReader.");
				mailingLabelsText.setFocusable(false);
				demoGroupBox.add(mailingLabelsText, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0,
																		   GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																		   new Insets(0, 0, 5, 0), 0, 0));

				//---- employeesReportRadio ----
				employeesReportRadio.setText("Employees Report");
				employeesReportRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				employeesReportRadio.setActionCommand("Employees Report");
				demoGroupBox.add(employeesReportRadio, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
																			  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			  new Insets(0, 0, 5, 5), 0, 0));

				//---- employeesReportText ----
				employeesReportText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				employeesReportText.setEditable(false);
				employeesReportText.setOpaque(false);
				employeesReportText.setText("Populates a table in a document with information from the database, also inserts images stored in a BLOB field into the document.");
				employeesReportText.setFocusable(false);
				demoGroupBox.add(employeesReportText, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0,
																			 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			 new Insets(0, 0, 5, 0), 0, 0));

				//---- salesInvoiceRadio ----
				salesInvoiceRadio.setText("Sales Invoice");
				salesInvoiceRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				salesInvoiceRadio.setActionCommand("Sales Invoice");
				demoGroupBox.add(salesInvoiceRadio, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
																		   GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																		   new Insets(0, 0, 5, 5), 0, 0));

				//---- salesInvoiceText ----
				salesInvoiceText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				salesInvoiceText.setEditable(false);
				salesInvoiceText.setOpaque(false);
				salesInvoiceText.setText("Creates an invoice with order header, details and summary. Shows how to mail merge data from multiple tables, format date and numeric fields and use merge regions to grow portions of the document.");
				salesInvoiceText.setFocusable(false);
				demoGroupBox.add(salesInvoiceText, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0,
																		  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																		  new Insets(0, 0, 5, 0), 0, 0));

				//---- productCatalogRadio ----
				productCatalogRadio.setText("Product Catalog");
				productCatalogRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				productCatalogRadio.setActionCommand("Product Catalog");
				demoGroupBox.add(productCatalogRadio, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
																			 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			 new Insets(0, 0, 5, 5), 0, 0));

				//---- productCatalogText ----
				productCatalogText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				productCatalogText.setEditable(false);
				productCatalogText.setOpaque(false);
				productCatalogText.setText("Creates a list of products grouped by product category. Shows how to create reports with master-child data, combine documents, move sections between documents and mail merge images stored in files outside of the database.");
				productCatalogText.setFocusable(false);
				demoGroupBox.add(productCatalogText, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0,
																			GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			new Insets(0, 0, 5, 0), 0, 0));

				//---- alternatingRowsRadio ----
				alternatingRowsRadio.setText("Alternating Rows");
				alternatingRowsRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				alternatingRowsRadio.setActionCommand("Alternating Rows");
				demoGroupBox.add(alternatingRowsRadio, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
																			  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			  new Insets(0, 0, 5, 5), 0, 0));

				//---- alternatingRowsText ----
				alternatingRowsText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				alternatingRowsText.setEditable(false);
				alternatingRowsText.setOpaque(false);
				alternatingRowsText.setText("Creates a list of products with alternating table row colors. Shows how to use DocumentBuilder inside mail merge event handler.");
				alternatingRowsText.setFocusable(false);
				demoGroupBox.add(alternatingRowsText, new GridBagConstraints(1, 5, 1, 1, 1.0, 0.0,
																			 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			 new Insets(0, 0, 5, 0), 0, 0));

				//---- documentBuilderRadio ----
				documentBuilderRadio.setText("Document Builder");
				documentBuilderRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				documentBuilderRadio.setActionCommand("Document Builder");
				demoGroupBox.add(documentBuilderRadio, new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0,
																			  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			  new Insets(0, 0, 5, 5), 0, 0));

				//---- documentBuilderText ----
				documentBuilderText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				documentBuilderText.setEditable(false);
				documentBuilderText.setOpaque(false);
				documentBuilderText.setText("Manipulates a document programmatically. Inserts formatted text, paragraphs, images, tables and other content into a Word document.");
				documentBuilderText.setFocusable(false);
				demoGroupBox.add(documentBuilderText, new GridBagConstraints(1, 6, 1, 1, 1.0, 0.0,
																			 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			 new Insets(0, 0, 5, 0), 0, 0));

				//---- stampAndWatermarkRadio ----
				stampAndWatermarkRadio.setText("Stamp and Watermark");
				stampAndWatermarkRadio.setFont(new Font("Tahoma", Font.PLAIN, 12));
				stampAndWatermarkRadio.setActionCommand("Stamp and Watermark");
				demoGroupBox.add(stampAndWatermarkRadio, new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0,
																				GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																				new Insets(0, 0, 0, 5), 0, 0));

				//---- stampAndWatermarkText ----
				stampAndWatermarkText.setFont(new Font("Tahoma", Font.PLAIN, 12));
				stampAndWatermarkText.setEditable(false);
				stampAndWatermarkText.setOpaque(false);
				stampAndWatermarkText.setText("Opens an existing document and inserts a watermark image and a footer with some information to indicate the document was processed by the system.");
				stampAndWatermarkText.setFocusable(false);
				demoGroupBox.add(stampAndWatermarkText, new GridBagConstraints(1, 7, 1, 1, 1.0, 0.0,
																			   GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																			   new Insets(0, 0, 0, 0), 0, 0));
			}
			contentPanel.add(demoGroupBox);

			//======== formatGroupBox ========
			{
				formatGroupBox.setBorder(new TitledBorder("Output format"));
				formatGroupBox.setPreferredSize(new Dimension(1503, 128));
				formatGroupBox.setLayout(new GridBagLayout());
				((GridBagLayout)formatGroupBox.getLayout()).columnWidths = new int[] {0, 138, 0};
				((GridBagLayout)formatGroupBox.getLayout()).rowHeights = new int[] {0, 0, 0, 0, 0, 0};
				((GridBagLayout)formatGroupBox.getLayout()).columnWeights = new double[] {0.0, 1.0, 1.0E-4};
				((GridBagLayout)formatGroupBox.getLayout()).rowWeights = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 1.0E-4};

				//---- docButton ----
				docButton.setText("DOC - Microsoft Office Word 2003 Document");
				docButton.setAlignmentX(1.0F);
				docButton.setAlignmentY(1.0F);
				docButton.setFont(new Font("Tahoma", Font.PLAIN, 12));
				docButton.setSelected(true);
				docButton.setActionCommand("doc");
				formatGroupBox.add(docButton, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
																	 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																	 new Insets(0, 0, 5, 5), 0, 0));

				//---- docxButton ----
				docxButton.setAlignmentX(1.0F);
				docxButton.setAlignmentY(1.0F);
				docxButton.setFont(new Font("Tahoma", Font.PLAIN, 12));
				docxButton.setText("DOCX - Microsoft Office 2007 Open XML");
				docxButton.setActionCommand("docx");
				formatGroupBox.add(docxButton, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
																	  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																	  new Insets(0, 0, 5, 5), 0, 0));

				//---- htmlButton ----
				htmlButton.setText("HTML/XHTML - XHTML 1.0 Transitional");
				htmlButton.setAlignmentX(1.0F);
				htmlButton.setAlignmentY(1.0F);
				htmlButton.setFont(new Font("Tahoma", Font.PLAIN, 12));
				htmlButton.setActionCommand("html");
				formatGroupBox.add(htmlButton, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
																	  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																	  new Insets(0, 0, 5, 5), 0, 0));

				//---- txtButton ----
				txtButton.setText("TXT - Plain Text");
				txtButton.setAlignmentX(1.0F);
				txtButton.setAlignmentY(1.0F);
				txtButton.setFont(new Font("Tahoma", Font.PLAIN, 12));
				txtButton.setActionCommand("txt");
				formatGroupBox.add(txtButton, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
																	 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																	 new Insets(0, 0, 5, 5), 0, 0));

				//---- pdfButton ----
				pdfButton.setText("PDF - Adobe Portable Document Format (experimental, very limited, using iText)");
				pdfButton.setAlignmentX(1.0F);
				pdfButton.setAlignmentY(1.0F);
				pdfButton.setFont(new Font("Tahoma", Font.PLAIN, 12));
				pdfButton.setActionCommand("pdf");
				formatGroupBox.add(pdfButton, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
																	 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
																	 new Insets(0, 0, 0, 5), 0, 0));
			}
			contentPanel.add(formatGroupBox);
		}
		contentPane.add(contentPanel, BorderLayout.CENTER);

		//======== buttonBar ========
		{
			buttonBar.setBorder(new EmptyBorder(5, 5, 5, 5));
			buttonBar.setLayout(new GridBagLayout());
			((GridBagLayout)buttonBar.getLayout()).columnWidths = new int[] {0, 85, 80};
			((GridBagLayout)buttonBar.getLayout()).columnWeights = new double[] {1.0, 0.0, 0.0};

			//---- saveButton ----
			saveButton.setText("Save");
			saveButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e) {
					saveButtonActionPerformed(e);
				}
			});
			buttonBar.add(saveButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
															 GridBagConstraints.CENTER, GridBagConstraints.BOTH,
															 new Insets(0, 0, 0, 5), 0, 0));

			//---- closeButton ----
			closeButton.setText("Close");
			closeButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e) {
					closeButtonActionPerformed(e);
				}
			});
			buttonBar.add(closeButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
															  GridBagConstraints.CENTER, GridBagConstraints.BOTH,
															  new Insets(0, 0, 0, 0), 0, 0));
		}
		contentPane.add(buttonBar, BorderLayout.SOUTH);
		setSize(665, 785);
		setLocationRelativeTo(null);

		//---- demoGroup ----
		demoGroup = new ButtonGroup();
		demoGroup.add(dinnerInvitationRadio);
		demoGroup.add(mailingLabelsRadio);
		demoGroup.add(employeesReportRadio);
		demoGroup.add(salesInvoiceRadio);
		demoGroup.add(productCatalogRadio);
		demoGroup.add(alternatingRowsRadio);
		demoGroup.add(documentBuilderRadio);
		demoGroup.add(stampAndWatermarkRadio);

		//---- formatGroup ----
		formatGroup = new ButtonGroup();
		formatGroup.add(docButton);
		formatGroup.add(docxButton);
		formatGroup.add(htmlButton);
		formatGroup.add(txtButton);
		formatGroup.add(pdfButton);
	}

	private JPanel preface;
	private JLabel label1;
	private JTextPane textPane1;
	private JTextPane textPane2;
	private JTextPane textPane3;
	private JPanel contentPanel;
	private JPanel demoGroupBox;
	private JRadioButton dinnerInvitationRadio;
	private JTextPane dinnerInvitationText;
	private JRadioButton mailingLabelsRadio;
	private JTextPane mailingLabelsText;
	private JRadioButton employeesReportRadio;
	private JTextPane employeesReportText;
	private JRadioButton salesInvoiceRadio;
	private JTextPane salesInvoiceText;
	private JRadioButton productCatalogRadio;
	private JTextPane productCatalogText;
	private JRadioButton alternatingRowsRadio;
	private JTextPane alternatingRowsText;
	private JRadioButton documentBuilderRadio;
	private JTextPane documentBuilderText;
	private JRadioButton stampAndWatermarkRadio;
	private JTextPane stampAndWatermarkText;
	private JPanel formatGroupBox;
	private JRadioButton docButton;
	private JRadioButton docxButton;
	private JRadioButton htmlButton;
	private JRadioButton txtButton;
	private JRadioButton pdfButton;
	private JPanel buttonBar;
	private JButton saveButton;
	private JButton closeButton;
	private ButtonGroup demoGroup;
	private ButtonGroup formatGroup;
}
