//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;


import com.aspose.words.Document;

import java.io.File;
import java.net.URL;
import java.sql.*;

/**
 * Base class for Aspose.Words.Demos classes that execute specific demos.
 */
public abstract class Demo
{
	/**
     * Called before launching any concrete demo-classes.
	 * We need application path so we can find the database and template documents.
     */
	public static void initDemo() throws Exception
	{
		//calculate common project dirs and files used by all demos.
		mProjectDir = getProjectDir();
		File baseDir = mProjectDir.getParentFile();
		mDocumentsDir = new File(baseDir, "Documents");
		mGeneratedDir = new File(mProjectDir, "generated");
		mDatabaseDir = new File(baseDir, "Database");
		mDatabase = new File(mDatabaseDir, "Northwind.mdb");

		//check all this exist.
		checkExistense();

		//wipe out the dir for generated files.
		if (mGeneratedDir.exists())
		{
			File[] files = mGeneratedDir.listFiles();
			for (int i = 0; i < files.length; i++)
				files[i].delete();
		}
		else
		{
			mGeneratedDir.mkdir();
		}

		//get an access to the database.
		mStatement = createStatement();
	}

	/**
	 * Calculates the root dir of the demo project.
	 *
	 * The demo project dir is "demos/Aspose.Words.Demos".
	 * It is proposed that compiled classes will be placed by compiler into "classes" subdir
	 * of the project dir, so Demo.class (this) will be placed into
	 * "demos/Aspose.Words.Demos/classes/com/aspose/words/demos/Demo.class".
	 */
	private static File getProjectDir()
	{
		URL url = Demo.class.getResource("Demo.class");

		String path = new File(url.getPath()).getAbsolutePath();
		String subPath = "\\classes\\com\\aspose\\words\\demos\\Demo.class";

		return new File(path.substring(0, path.length() - subPath.length()));
	}

	/**
	 * Generally: wrong project path --> wrong dependent dirs and files.
	 */
	private static void checkExistense()
	{
		if (!mProjectDir.exists())
			throw new IllegalArgumentException("Can't find the Project dir: " + mProjectDir.getAbsolutePath());

		if (!mDocumentsDir.exists())
			throw new IllegalArgumentException("Can't find the template documents dir: " + mDocumentsDir.getAbsolutePath());

		if (!mDatabaseDir.exists())
			throw new IllegalArgumentException("Can't find the database dir: " + mDatabaseDir.getAbsolutePath());

		if (!mDatabase.exists())
			throw new IllegalArgumentException("Can't find the database file: " + mDatabase.getAbsolutePath());
	}

	/**
	 * Utility function that creates a connection to the Database.
	 */
	private static Statement createStatement() throws Exception
	{
		//Load a DB driver that is used by the demos
		Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		//Compose connection string.
		String connectionString = "jdbc:odbc:DRIVER={Microsoft Access Driver (*.mdb)};" +
								  "DBQ=" + mDatabase + ";UID=Admin";
		//DSN-less DB connection.
		Connection connection = DriverManager.getConnection(connectionString);

		return connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
	}

	public static void closeDB() throws SQLException
	{
		if (mStatement != null)
			mStatement.close();
	}

	/**
     * Called by concrete demo-class. Calculates paths to files used solely by this concrete demo.
     */
    protected void init(String demoName)
	{
		//calculate files used solely by this concrete demo.
		mDemoName = demoName;
		mTemplate = new File(mDocumentsDir, mDemoName + ".doc");
		if (!mTemplate.exists())
			throw new IllegalArgumentException("Can't find the template document file: " + mTemplate.getAbsolutePath());
	}

    /**
     * Executes the demo and returns a generated document. Implement in the derived classes.
     */
    public abstract Document execute() throws Exception;

	/**
	 * Executes a query to the demo database and returns the result in a ResultSet.
	 */
	protected ResultSet executeQuery(String query) throws SQLException
	{
		return mStatement.executeQuery(query);
	}

	//JAVA-added
	/**
	 * Returns an demo name.
	 */
	String getDemoName()
	{
		return mDemoName;
	}

	//JAVA-added
	/**
	 * Returns a full path where the template document used by this demos is stored.
	 */
	String getPathToTemplateDoc()
	{
		return mTemplate.getAbsolutePath();
	}

	//JAVA-added
	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	String getPathToGeneratedDoc()
	{
		return new File(mGeneratedDir, mDemoName + ".doc").getAbsolutePath();
	}

	//JAVA-added
	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	String getPathToGeneratedHtml()
	{
		return new File(mGeneratedDir, mDemoName + ".html").getAbsolutePath();
	}

	//JAVA-added
	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	String getPathToGeneratedDocx()
	{
		return new File(mGeneratedDir, mDemoName + ".docx").getAbsolutePath();
	}

	//JAVA-added
	/**
	 * Returns a full path where the document generated by this demo should be stored.
	 */
	String getPathToGeneratedTxt()
	{
		return new File(mGeneratedDir, mDemoName + ".txt").getAbsolutePath();
	}

	//JAVA-added
	/**
	 * Returns a root dir of this Aspose.Words release.
	 * (the dir where unzipped downloaded Aspose.Words distributive).
	 * Probably in that dir will be placed license file.
	 *
	 * Note: Can be called only after the demo initialization (initDemo() method).
	 */
	static File getRootDistributiveDir()
	{
		assert mProjectDir != null;
		return mProjectDir.getParentFile().getParentFile();
	}

	/**
	 * Project dir. Something like: 'Aspose.Words.Java installation path'\\demos\\Aspose.Words.Demos
	 */
	protected static File mProjectDir;
	/**
	 * Directory for template documents used by the demos: demos\\Documents.
	 */
	protected static File mDocumentsDir;
	/**
	 * Directory for documents generated by the demos: demos\\generated.
	 */
	protected static File mGeneratedDir;
	/**
	 * Directory for the Databese used by the demos: demos\\Database.
	 */
	protected static File mDatabaseDir;
	/**
	 * Databese file used by the demos: demos\\Database\\Northwind.mdb.
	 */
	protected static File mDatabase;
	/**
	 * Short name of the template document for the concrete demo.
	 */
	protected String mDemoDocName;
	/**
	 * The name of this demo. Template document and all generated documents use this name.
	 */
	protected String mDemoName;
	/**
	 * Template document file used by the concrete demo.
	 */
	protected File mTemplate;

	private static Statement mStatement;
}
