//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import java.io.*;
import java.util.*;

/**
 * Inner class for conversation from the Word document into the Adobe Reader document using Apose.Words and iText
 */
public class Doc2PdfConverter extends com.aspose.words.DocumentVisitor
{
    // Temporary table
    static class TemporaryTable
    {
        // Maximum column count in the current row
        private int mMaximumColumnCount                  = 0;
        // Current column count in the current row
        private int mCurrentColumnCount                  = 0;
        // Are we now in the cell ?
        private boolean mInCell                         = false;
        // Cell stack
        private Stack mCellStack                        = new Stack();
    }

    // Temporary Paragraph
    static class TemporaryParagraph
    {
        // Maximum chunk's font size
        private float mMaximumFontSize                   = 0;
        // Paragraph
        private com.lowagie.text.Paragraph mParagraph    = null;
    }

    // Type of the field
    static class FieldType
    {
        private static String PAGE = "PAGE";
    }

    // PDF document
    private com.lowagie.text.Document mPdfDocument              = null;
    // For future use
    //private com.lowagie.text.pdf.PdfWriter mPdfWriter           = null;
    // PDF file name
    private OutputStream mOutputStream                          = null;
    // If we are in not needed element we just skip this information
    private boolean mIsSkipText                                 = false;
    // Are we now in paragraph ?
    private boolean mInParagraph                                = false;
    // Is the section the first section of the document ?
    private boolean mIsFirstSection                             = true;
    // Are we now in HeaderFooter element ?
    private boolean mInHeaderFooter                             = false;
    private boolean mNumberedHeaderFooter                       = false;
    // It is possible to remove stack and use only variable for the current paragraph
    private Stack mParaStack                                    = new Stack();
    // Stack of a temporary table
    private Stack mTableStack                                   = new Stack();
    // Current temporary table
    private TemporaryTable mCurrentTable                        = null;
    // Current list
    private com.lowagie.text.List mCurrentList                  = null;
    // Current HeadeFooter
    private com.lowagie.text.HeaderFooter mCurrentHeaderFooter  = null;
    // For future use
    // private com.lowagie.text.List mParentList                   = null;
    // private int mCurrentListLevel                               = -1;

    //-----------------------------------------------------------------------------
    public Doc2PdfConverter(OutputStream outputStream)
    {
        this.mOutputStream = outputStream;
    }

    /**
     * Called when visiting of a Section node is started in the document.
     */
    public int visitSectionStart(com.aspose.words.Section wordSection) throws Exception
    {
        if(wordSection != null)
        {
            // Get word's section information and set PDF section information
            com.aspose.words.PageSetup ps = wordSection.getPageSetup();

            float pageWidth    = (float)ps.getPageWidth();
            float pageHeight   = (float)ps.getPageHeight();
            com.lowagie.text.Rectangle pageSize = new com.lowagie.text.Rectangle(pageWidth, pageHeight);
            float leftMargin   = (float)ps.getLeftMargin();
            float rightMargin  = (float)ps.getRightMargin();
            float topMargin    = (float)ps.getTopMargin();
            float bottomMargin = (float)ps.getTopMargin();

            if(mIsFirstSection)
            {
                // If this is the first section in the word document, we create new PDF document and set information about this document
                mIsFirstSection = false;
                mPdfDocument = new com.lowagie.text.Document(pageSize, leftMargin, rightMargin, topMargin, bottomMargin);
                /*mPdfWriter = */com.lowagie.text.pdf.PdfWriter.getInstance(mPdfDocument, mOutputStream);
            }
            else
            {
                // Next section. We create new page and also set all need information
                mPdfDocument.setPageSize(pageSize);
                mPdfDocument.setMargins(leftMargin, rightMargin, topMargin, bottomMargin);
                mPdfDocument.newPage();
            }
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Document node is ended in the document.
     */
    public int visitDocumentEnd(com.aspose.words.Document wordDocument) throws Exception
    {
        // Close PDF document
        if(mPdfDocument != null)
        {
            mPdfDocument.close();
            mPdfDocument = null;
        }
        return com.aspose.words.VisitorAction.STOP;
    }

    /**
     * Called when visiting of a Body node is started in the document.
     */
    public int visitBodyStart(com.aspose.words.Body wordBody) throws Exception
    {
        mPdfDocument.open();
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Paragraph node is started in the document.
     */
    public int visitParagraphStart(com.aspose.words.Paragraph wordParagraph) throws Exception
    {
        if(wordParagraph != null)
        {
            mInParagraph = true;
            // We are now in a word paragraph
            // Create new PDF paragraph and set formating and other information in it
            com.aspose.words.ParagraphFormat wordFormat = wordParagraph.getParagraphFormat();
            com.lowagie.text.Paragraph para = new com.lowagie.text.Paragraph();
            setFormat(para, wordFormat);
            TemporaryParagraph tempPara = new TemporaryParagraph();
            tempPara.mParagraph = para;
            mParaStack.push(tempPara);
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Paragraph node is ended in the document.
     */
    public int visitParagraphEnd(com.aspose.words.Paragraph wordParagraph) throws Exception
    {
        if(wordParagraph != null && !mParaStack.empty())
        {
            TemporaryParagraph tempPara = (TemporaryParagraph)mParaStack.pop();
            com.lowagie.text.Paragraph para = tempPara.mParagraph;
            com.aspose.words.ParagraphFormat wordFormat = wordParagraph.getParagraphFormat();

            // Check for an empty Paragraph
            if(para.size() <= 0)
                para.add(com.lowagie.text.Chunk.NEWLINE);
            else
            {
                int lineSpacingRule = wordFormat.getLineSpacingRule();
                float lineSpacing = (float)wordFormat.getLineSpacing();
                float fontSize = tempPara.mMaximumFontSize;
                if(lineSpacingRule==com.aspose.words.LineSpacingRule.EXACTLY)
                    fontSize = lineSpacing;
                else if(lineSpacingRule==com.aspose.words.LineSpacingRule.MULTIPLE && lineSpacing>fontSize)
                    fontSize = lineSpacing;
                para.setLeading(fontSize);
            }

            // Check whether the paragraph is a list item, so we add it to the list, not to the PDF or cell
            if(wordFormat.isListItem())
            {
                //com.aspose.words.ListFormat listFormat = wordParagraph.getListFormat();
                // int listLevel = listFormat.getListLevelNumber();

                com.lowagie.text.ListItem listItem = new com.lowagie.text.ListItem(para);
                setFormat(listItem, wordFormat);
                // create new list
                if(mCurrentList == null)
                {
                    com.lowagie.text.List list = new com.lowagie.text.List(false, false, 10);
                    list.setIndentationLeft((float)wordFormat.getLeftIndent());
                    list.setIndentationRight((float)wordFormat.getRightIndent());
                    mCurrentList = list;
                }
                mCurrentList.add(listItem);

                // For future use
                /*
                // Create new list
                if(listLevel != mCurrentListLevel)
                {
                    mCurrentListLevel = listLevel;
                    List list = new List(false, false, format.getFirstLineIndent());
                    list.setIndentationLeft((float)format.getLeftIndent());
                    list.setIndentationRight((float)format.getRightIndent());
                    mCurrentList = list;
                    if(mParentList != null)
                        mParentList.add(mCurrentList);
                    else
                        mParentList = list;
                }
                mCurrentList.add(listItem);
                */
            }

            // Last paragraph in the list
            if(wordParagraph.isEndOfCell() || wordParagraph.isEndOfDocument() || wordParagraph.isEndOfSection()
               ||!wordFormat.isListItem())
            {
                com.lowagie.text.Element currentElement;

                // For future use
                /*
                if(mParentList == null)
                    currentElement = para;
                else
                {
                    currentElement = mParentList;
                    mCurrentListLevel = -1;
                    mParentList = mCurrentList = null;
                }
                */
                if(mCurrentList == null)
                    currentElement = para;
                else
                {
                    currentElement = mCurrentList;
                    mCurrentList = null;
                }

                // add to the cell if we're processing table's cell right now, otherwise add to PDF
                if(mCurrentTable != null && mCurrentTable.mInCell && !mCurrentTable.mCellStack.empty())
                    ((com.lowagie.text.Cell)mCurrentTable.mCellStack.peek()).addElement(currentElement);
                else if(mInHeaderFooter)
                {
                    if(mCurrentList == null)
                        mCurrentHeaderFooter = new com.lowagie.text.HeaderFooter(para, mNumberedHeaderFooter);
                }
                else
                    mPdfDocument.add(currentElement);
                if(mParaStack.empty())
                    mInParagraph = false;
            }
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when a FieldStart node is encountered in the document.
     */
    public int visitFieldStart(com.aspose.words.FieldStart wordFieldStart) throws Exception
    {
        // In Microsoft Word, a field code (such as "MERGEFIELD FieldName") follows
        // after a field start character. We want to skip field codes and output field
        // result only, therefore we use a flag to suspend the output while inside a field code.
        //
        // Note this is a very simplistic implementation and will not work very well
        // if you have nested fields in a document.
        // Fields don't work yet
        int fieldType = wordFieldStart.getFieldType();
        if(fieldType==com.aspose.words.FieldType.FIELD_PAGE)
            mNumberedHeaderFooter = true;

        /*
        com.lowagie.text.pdf.TextField textField =  new com.lowagie.text.pdf.TextField(mPdfWriter, new com.lowagie.text.Rectangle(200, 600, 300, 620), "field1");
        com.lowagie.text.pdf.PdfFormField form = textField.getTextField();
        mPdfWriter.addAnnotation(form);
        */
        mIsSkipText = true;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when a FieldSeparator node is encountered in the document.
     */
    public int visitFieldSeparator(com.aspose.words.FieldSeparator wordFieldSeparator) throws Exception
    {
        // Once reached a field separator node, we enable the output because we are
        // now entering the field result nodes.
        mIsSkipText = true;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when a FieldEnd node is encountered in the document.
     */
    public int visitFieldEnd(com.aspose.words.FieldEnd wordFieldEnd) throws Exception
    {
        // Make sure we enable the output when reached a field end because some fields
        // do not have field separator and do not have field result.
        mIsSkipText = false;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Footnote node is started in the document.
     */
    public int visitFootnoteStart(com.aspose.words.Footnote wordFootnote) throws Exception
    {
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Footnote node is ended in the document.
     */
    public int visitFootnoteEnd(com.aspose.words.Footnote wordFootnote) throws Exception
    {
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a HeaderFooter node is started in the document.
     */
    public int visitHeaderFooterStart(com.aspose.words.HeaderFooter wordHeaderFooter) throws Exception
    {
        mInHeaderFooter = true;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a HeaderFooter node is ended in the document.
     */
    public int visitHeaderFooterEnd(com.aspose.words.HeaderFooter wordHeaderFooter) throws Exception
    {
        mInHeaderFooter = false;
        if(wordHeaderFooter != null && mCurrentHeaderFooter != null)
        {
            int type = wordHeaderFooter.getHeaderFooterType();
            switch(type)
            {
                case com.aspose.words.HeaderFooterType.HEADER_PRIMARY:
                //case com.aspose.words.HeaderFooterType.HEADER_EVEN:
                //case com.aspose.words.HeaderFooterType.HEADER_FIRST:
                    mPdfDocument.setHeader(mCurrentHeaderFooter);
                break;
                case com.aspose.words.HeaderFooterType.FOOTER_PRIMARY:
                //case com.aspose.words.HeaderFooterType.FOOTER_EVEN:
                //case com.aspose.words.HeaderFooterType.FOOTER_FIRST:
                    mPdfDocument.setFooter(mCurrentHeaderFooter);
                break;
            }
            mNumberedHeaderFooter = false;
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    //================ Tables ==================

    /**
     * Called when visiting of a Table node is started in the document.
     */
    public int visitTableStart(com.aspose.words.Table wordTable) throws Exception
    {
        // Create new temporary table so we can calculate number of columns
        TemporaryTable temporaryTable = new TemporaryTable();
        temporaryTable.mMaximumColumnCount = 0;
        mTableStack.push(temporaryTable);
        mCurrentTable = temporaryTable;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Table node is ended in the document.
     */
    public int visitTableEnd(com.aspose.words.Table wordTable) throws Exception
    {
        if(mCurrentTable != null && mCurrentTable.mMaximumColumnCount>0)
        {
            // Create new PDF table
            com.lowagie.text.Table table = new com.lowagie.text.Table(mCurrentTable.mMaximumColumnCount);
            // Add all cells from the stack to the current table
            int stackSize = mCurrentTable.mCellStack.size();
            for(int i=0; i<stackSize; ++i)
                table.addCell((com.lowagie.text.Cell) mCurrentTable.mCellStack.get(i));
            mCurrentTable.mCellStack.clear();
            mCurrentTable.mCellStack = null;
            // Remove current table from the stack
            if(!mTableStack.empty())
                mTableStack.pop();
            // Take previous table
            if(!mTableStack.empty())
            {
                mCurrentTable = (TemporaryTable)mTableStack.peek();
                // Add current table to the previous table if it exists
                // else add current table to the PDF document
                if(mCurrentTable.mInCell && !mCurrentTable.mCellStack.empty())
                    ((com.lowagie.text.Cell)mCurrentTable.mCellStack.peek()).addElement(table);
                else if(!mInHeaderFooter)
                    mPdfDocument.add(table);
            }
            else if(!mInHeaderFooter)
            {
                mCurrentTable = null;
                mPdfDocument.add(table);
            }
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Row node is started in the document.
     */
    public int visitRowStart(com.aspose.words.Row wordRow) throws Exception
    {
        // calculate number of columns
        if(mCurrentTable != null)
            mCurrentTable.mCurrentColumnCount = 0;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Row node is ended in the document.
     */
    public int visitRowEnd(com.aspose.words.Row wordRow) throws Exception
    {
        // calculate number of columns
        if(mCurrentTable != null && mCurrentTable.mCurrentColumnCount > mCurrentTable.mMaximumColumnCount)
            mCurrentTable.mMaximumColumnCount = mCurrentTable.mCurrentColumnCount;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Cell node is started in the document.
     */
    public int visitCellStart(com.aspose.words.Cell wordCell) throws Exception
    {
        if(wordCell != null)
        {
            if(mCurrentTable != null)
            {
                com.aspose.words.CellFormat format = wordCell.getCellFormat();
                ++mCurrentTable.mCurrentColumnCount;
                com.lowagie.text.Cell cell = new com.lowagie.text.Cell();

                cell.setVerticalAlignment(convertCellAllignment(format.getVerticalAlignment()));
                com.aspose.words.Shading shading = format.getShading();
                // Doesn't work, shading.Texture() is always 0
                int texture = shading.getTexture();
                if(texture!=com.aspose.words.TextureIndex.TEXTURE_NONE)
                    cell.setBackgroundColor(shading.getBackgroundPatternColor());
                // collspan and rowspan don't work yet
                //int hm = format.getHorizontalMerge();
                //int vm = format.getVerticalMerge();
                //cell.setRowspan(vm+1);
                //cell.setColspan(hm+1);
                mCurrentTable.mCellStack.push(cell);
                mCurrentTable.mInCell = true;
            }
        }
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Called when visiting of a Cell node is ended in the document.
     */
    public int visitCellEnd(com.aspose.words.Cell wordCell) throws Exception
    {
        if(mCurrentTable != null)
            mCurrentTable.mInCell = false;
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    //=============== End Tables =================

    /**
     * Called when a Run node is encountered in the document.
     */
    public int visitRun(com.aspose.words.Run wordRun) throws Exception
    {
        // We've got text for the element, so add it to the current element
        if(wordRun != null)
            AddChunk(wordRun);
        return com.aspose.words.VisitorAction.CONTINUE;
    }

    /**
     * Add chunk to the current element
     */
    private void AddChunk(com.aspose.words.Run wordRun) throws Exception
    {
        if(!mIsSkipText)
        {
            // Get item's text
            StringBuffer text = new StringBuffer(wordRun.getText());
            // Add new page if there is page break character in the text
            int indexOfPageBreak = -1;
            while((indexOfPageBreak=text.indexOf(com.aspose.words.ControlChar.PAGE_BREAK, indexOfPageBreak)) != -1)
            {
                text.deleteCharAt(indexOfPageBreak);
                mPdfDocument.newPage();
            }
            String chunkText = text.toString();
            chunkText = chunkText.replace(com.aspose.words.ControlChar.LINE_BREAK_CHAR, '\n');

            // Create new chunk and set chunk's format
            com.lowagie.text.Chunk chunk = new com.lowagie.text.Chunk(chunkText);
            com.aspose.words.Font wordFont = wordRun.getFont();
            chunk.setFont(convertFont(wordFont));
            // 1. If we are now in a paragraph, add chunk to the paragraph
            // 2. If we are in a table's cell, add chunk to the cell
            // 3. Else we just add text to the PDF document
            if(mInParagraph && !mParaStack.empty())
            {
                TemporaryParagraph tempPara = (TemporaryParagraph)mParaStack.peek();
                float wordFontSize = (float)wordFont.getSize();
                if(wordFontSize>tempPara.mMaximumFontSize)
                    tempPara.mMaximumFontSize = wordFontSize;
                tempPara.mParagraph.add(chunk);
            }
            else if(mCurrentTable!=null && mCurrentTable.mInCell && !mCurrentTable.mCellStack.empty())
                ((com.lowagie.text.Cell)mCurrentTable.mCellStack.peek()).add(chunk);
            else
                mPdfDocument.add(chunk);
        }
    }

    /**
     * Set format of the current PDF paragraph using information from the Word paragraph
     */
    private static void setFormat(com.lowagie.text.Paragraph para, com.aspose.words.ParagraphFormat format) throws Exception
    {
        para.setAlignment           (convertParagraphAllignment(format.getAlignment()));
        para.setIndentationLeft     ((float)format.getLeftIndent());
        para.setIndentationRight    ((float)format.getRightIndent());
        para.setSpacingAfter        ((float)format.getSpaceAfter());
        para.setSpacingBefore       ((float)format.getSpaceBefore());
        para.setKeepTogether        (format.getKeepTogether());
        // setFirstLineIndent works only in ColumnText
        // para.setFirstLineIndent     ((float)format.getFirstLineIndent());
    }

    /**
     * Convert text style from the Word format to the PDF
     */
    private static int convertStyle(com.aspose.words.Font docFont) throws Exception
    {
        int style = com.lowagie.text.Font.NORMAL;
        if(docFont.getBold())
            style |= com.lowagie.text.Font.BOLD;
        if(docFont.getItalic())
            style |= com.lowagie.text.Font.ITALIC;
        if(docFont.getUnderline()!=com.aspose.words.Underline.NONE)
            style |= com.lowagie.text.Font.UNDERLINE;
        return style;
    }

    /**
     * Convert text allignment from the Word format to the PDF
     */
    private static int convertParagraphAllignment(int docAllignment)
    {
        switch(docAllignment)
        {
            case com.aspose.words.ParagraphAlignment.CENTER:
                return com.lowagie.text.Element.ALIGN_CENTER;
            case com.aspose.words.ParagraphAlignment.DISTRIBUTED:
                return com.lowagie.text.Element.ALIGN_JUSTIFIED_ALL;
            case com.aspose.words.ParagraphAlignment.JUSTIFY:
                return  com.lowagie.text.Element.ALIGN_JUSTIFIED;
            case com.aspose.words.ParagraphAlignment.LEFT:
                return com.lowagie.text.Element.ALIGN_LEFT;
            case com.aspose.words.ParagraphAlignment.RIGHT:
                return com.lowagie.text.Element.ALIGN_RIGHT;
            default:
                return com.lowagie.text.Element.ALIGN_UNDEFINED;
        }
    }

    /**
     * Convert text allignment for a cell from the Word format to the PDF
     */
    private static int convertCellAllignment(int docAllignment)
    {
        switch(docAllignment)
        {
            case com.aspose.words.CellVerticalAlignment.CENTER:
                return com.lowagie.text.Cell.ALIGN_CENTER;
            case com.aspose.words.CellVerticalAlignment.BOTTOM:
                return com.lowagie.text.Cell.ALIGN_BOTTOM;
            case com.aspose.words.CellVerticalAlignment.TOP:
                return  com.lowagie.text.Cell.ALIGN_TOP;
            default:
                return com.lowagie.text.Cell.ALIGN_UNDEFINED;
        }
    }

    /**
     * Convert font from the Word format to the PDF
     */
    private static com.lowagie.text.Font convertFont(com.aspose.words.Font wordFont) throws Exception
    {
        java.awt.Color color = wordFont.getColor();
        float size = (float)wordFont.getSize();
        int style = convertStyle(wordFont);
        String fontName = wordFont.getName();
        int font = com.lowagie.text.Font.TIMES_ROMAN;
        if(fontName.indexOf("Courier")!=-1)
            font = com.lowagie.text.Font.COURIER;
        return new com.lowagie.text.Font(font, size, style, color);
    }
}