//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.demos;

import com.aspose.words.*;

import java.awt.*;

/**
 * Manipulates a document programmatically. Inserts formatted text, paragraphs,
 * images, tables and other content into a Word document.
 */
public class DocumentBuilderDemo extends Demo
{
	public DocumentBuilderDemo()
	{
		init("DocumentBuilderDemo");
	}

    public Document execute() throws Exception
    {
        //Open the template document
		Document doc = new Document(getPathToTemplateDoc());

        //Once the builder is created, its cursor is positioned at the beginning of the document.
        DocumentBuilder builder = new DocumentBuilder(doc);

        //Specify font formatting before adding text.
        builder.getFont().setSize(16);
        builder.getFont().setBold(true);
        builder.getFont().setColor(Color.BLUE);
        builder.getFont().setName("Arial");
        builder.setUnderline(Underline.DASH);
        builder.write("This text");
        builder.setUnderline(Underline.NONE);
        builder.write(" is inserted at the beginning of the document and demonstrates ");

        //Set shading and border around a run of text, then clear the border.
        builder.getFont().getShading().setForegroundPatternColor(Color.CYAN);
        builder.getFont().getShading().setTexture(TextureIndex.TEXTURE_SOLID);
        builder.write("various font");
        builder.getFont().getShading().clearFormatting();
        builder.getFont().getBorder().setLineStyle(LineStyle.SINGLE);
        builder.write(" formatting ");
        builder.getFont().getBorder().clearFormatting();
        builder.writeln("options.");
        builder.insertParagraph();

        //Move the insertion cursor to a merge field.
        builder.moveToMergeField("MyInsertionPlace1");

        //Specify paragraph formatting before calling InsertParagraph or Writeln.
        builder.getParagraphFormat().getBorders().get(BorderType.TOP).setLineStyle(LineStyle.DOUBLE);
        builder.getParagraphFormat().getBorders().get(BorderType.TOP).setLineWidth(2);
        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);
        builder.getParagraphFormat().getShading().setForegroundPatternColor(new Color(240, 255, 255));
        builder.getParagraphFormat().getShading().setTexture(TextureIndex.TEXTURE_SOLID);
        builder.getParagraphFormat().setLeftIndent(2 * 72);	//2 inchdes by 72 points.
        builder.getParagraphFormat().setSpaceBefore(12);
        builder.getParagraphFormat().setSpaceAfter(12);
        builder.getFont().setName("Arial");
        builder.getFont().setSize(12);
        builder.write("This fragment demonstrates use of paragraph formatting to specify ");
        builder.writeln("alignment, spacing, borders and shading.");
        builder.getParagraphFormat().clearFormatting();

        //Create a fancy looking table inside the document.
        buildChessBoard(builder);

        //Insert another section and demonstrate page setup properties.
        builder.getFont().setSize(12);
        builder.insertBreak(BreakType.SECTION_BREAK_NEW_PAGE);
        builder.getPageSetup().setPaperSize(PaperSize.A5);
        builder.getPageSetup().setOrientation(Orientation.LANDSCAPE);
        builder.writeln("This text is in a section with different page size and orientation.");

		//Insert html - demonstrate html import.
		builder.insertHtml("<p>This text is <B><I>Bold Italic</B></I> inserted from html formatted string.");

        //Move to a position in the header.
        builder.moveToMergeField("MyHeader");
        builder.getFont().clearFormatting();
        builder.getFont().setItalic(true);
        builder.write("This text is inserted into the header.");

        //Don't need merge fields in the document anymore.
        doc.getMailMerge().deleteFields();

		return doc;
    }

    /**
     * Creates a chess like table to test borders and shading.
     */
    public void buildChessBoard(DocumentBuilder builder) throws Exception
    {
        //Reset to default font and paragraph formatting.
        builder.getFont().clearFormatting();
        builder.getParagraphFormat().clearFormatting();

        builder.insertParagraph();
        builder.writeln("This shows how to build a table in a document.");

        builder.getFont().setSize(6);
        builder.getFont().setName("Tahoma");
        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);

        //It is possible to change row format before (like here) and also during creation of the row.
        builder.getRowFormat().setHeight(0.4 * 72);	//0.4"
        builder.getRowFormat().setHeightRule(HeightRule.EXACTLY);

        //It is possible to change cell format before (like here) and also during creation of the cell.
        builder.getCellFormat().setWidth(0.4 * 72);	//0.4"
        builder.getCellFormat().setVerticalAlignment(CellVerticalAlignment.CENTER);

        BorderCollection borders = builder.getCellFormat().getBorders();
        borders.setLineStyle(LineStyle.SINGLE);
        borders.setLineWidth(2);
        borders.setColor(Color.BLUE);

        for (int y = 0; y < 8; y++)
        {
            for (int x = 0; x < 8; x++)
            {
                builder.insertCell();
                builder.write("X" + x + ", Y" + y);

                //This alternatives cell backgrounds.
                boolean isBlack = (((x + y) & 0x01) != 0);
                builder.getCellFormat().getShading().setBackgroundPatternColor(
                    (isBlack) ? new Color(144, 238, 144) : new Color(255, 255, 224));
            }
            builder.endRow();
        }
        builder.endTable();
    }
}
