//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////

package com.aspose.words.demos;

import com.aspose.words.*;

import java.io.File;
import java.sql.ResultSet;

/**
 * Creates a list of products grouped by product category. Shows how to create reports with master-child
 * data, combine documents, move sections between documents and mail merge images stored in files outside
 * of the database.
 */
public class ProductCatalogDemo extends Demo implements MergeImageFieldEventHandler
{
	public ProductCatalogDemo()
	{
		init("ProductCatalogDemo");
	}

	public Document execute() throws Exception
	{
		//Open the template document
		Document srcDoc = new Document(getPathToTemplateDoc());//JAVA-changed

		//Create a document where all product categories will be appended to.
		//We could have just created a new blank document as a destination, but we probably
		//want to keep the page setup and styles designed in our report so we just make a
		//copy of the template and clear its content.
		Document dstDoc = srcDoc.deepClone();
		dstDoc.getSections().clear();

		//Product catalog is an example of master-child report (Category is master and Product is child)
		//and calls for use of nested mail merge regions, but we cannot do this in the current
		//version of Aspose.Words.
		//
		//We work around by handling the loop for the categories "manually" so we can get
		//away only with one merge region for products within a category.
		for (int i = 1; i < 8; i++)
		{
			ResultSet categories = super.executeQuery("SELECT * FROM Categories WHERE CategoryID = " + i);

			//Each category needs a complete copy of the category template.
			//Making a copy in memory is much faster than opening the document every time from a file.
			Document categoryDoc = srcDoc.deepClone();

			//Attach an event handler that will be called for an Image merge field.
			//We need this because the database contains only short file names of the images
			//and Aspose.Words mail merge engine will need some help to find and load the images.
			categoryDoc.getMailMerge().addMergeImageFieldEventHandler(this);

			//There are some category header fields, fill them using normal mail merge.
			categoryDoc.getMailMerge().execute(categories);

			//Retrieve products for this category from the database and fill the products table in the report.
			ResultSet products = super.executeQuery("SELECT * FROM Products WHERE CategoryID = " + i);
			categoryDoc.getMailMerge().executeWithRegions(products);

			//The last thing to do is to add just generated category document to the complete report.
			appendDoc(dstDoc, categoryDoc);
		}

		//Add front and back cover pages to the report.
		//This demonstrates a general approach of creating documents  that is possible
		//by moving, copying or removing document sections.
		Document coverDoc = new Document(new File(mDocumentsDir, "ProductCatalogDemoCover.doc").getAbsolutePath());

		//Here we import the required sections and add them to the destination document.
		Node frontPage = dstDoc.importNode(coverDoc.getSections().get(0), true);
		dstDoc.prependChild(frontPage);

		Node orderPage = dstDoc.importNode(coverDoc.getSections().get(1), true);
		dstDoc.appendChild(orderPage);

		return dstDoc;
	}

	/**
	 * Called for every Image:[FieldName] merge field in the document.
	 */
	public void mergeImageField(Object sender, MergeImageFieldEventArgs e)
	{
		if ("PictureFileName".equals(e.getFieldName()))
		{
			//You can do a number of things in this event handler, this is just one scenario.
			//In our case the field in the database contains a file name (without path information)
			//of the image of the product category.
			String shortFileName = (String) e.getFieldValue();
			//We know that the image files are stored in the same folder as the database and
			//so we create the full file name.
			//Here, we pass the image file name to Aspose.Words mail merge engine.
			//Alternatively, we could have passed an Image Object or a Stream by
			//e.setImage(BufferedImage) or e.setImageStream(InputStream) setters.
			e.setImageFileName( new File(mDatabaseDir, shortFileName).getAbsolutePath());
		}
	}

	/**
	 * Copies all sections from one document to another.
	 */
	private void appendDoc(Document dstDoc, Document srcDoc) throws Exception
	{
		for (int i = 0; i < srcDoc.getSections().getCount(); i++)
		{
			//First need to import the section into the destination document,
			//this translates any document specific lists or styles.
			Section section = (Section)dstDoc.importNode(srcDoc.getSections().get(i), true);
			dstDoc.getSections().add(section);
		}
	}
}
