//////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Aspose Pty Ltd. All Rights Reserved.
//
// This file is part of Aspose.Words. The source code in this file
// is only intended as a supplement to the documentation, and is provided
// "as is", without warranty of any kind, either expressed or implied.
//////////////////////////////////////////////////////////////////////////
package com.aspose.words.documentexplorer;

import com.aspose.words.*;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import java.lang.reflect.*;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/// <summary>
/// Base class to provide GUI representation for document nodes.
/// </summary>
public class Item
{
    //java-changed: Variables moved to the top of the class, to resolve forward references from the static ctor
    private Node mNode;
    private DefaultMutableTreeNode mTreeNode;
    private ImageIcon mIcon;

    private static ArrayList<java.lang.reflect.Field> mControlCharFields;
    private static Map<Integer, String> mNodeTypes;
    private static Map<Integer, String> mHeaderFooterTypes;
    private static Map<String, String> mItemSet;
    private static ArrayList mIconNames = new ArrayList();

    /// <summary>
    /// Creates Item for the document node.
    /// </summary>
    /// <param name="node">Document node which this Item will represent.</param>
    public Item(Node node)
    {
        mNode = node;
    }

    /// <summary>
    /// Document node that this Item represents.
    /// </summary>
    public Node getNode()
    {
        return mNode;
    }

    /// <summary>
    ///  DisplayName for this Item. Can be customized by overriding in inheriting classes.
    /// </summary>
    public String getName() throws Exception
	{
		return getNodeTypeString(mNode);//java-changed: converts integer node type to string
	}

    /// <summary>
    /// Text contained by the corresponding document node.
    /// </summary>
    public String getText() throws Exception
    {
        String text = mNode.getText();

        // All control characters are converted to human readable form.
        // E.g. [!PageBreak!], [!ParagraphBreak!], etc.
        for (Field fieldInfo : mControlCharFields)
        {
            if (fieldInfo.getType() == char.class && Modifier.isStatic(fieldInfo.getModifiers()))
            {
                Character ch = fieldInfo.getChar(null);
                text = text.replace(ch.toString(), java.text.MessageFormat.format("[!{0}!]", fieldInfo.getName().replace("_CHAR", "")));
            }
        }

        // all break chars should be supplemented with line feeds
        text = text.replace("BREAK!]", "BREAK!]\r\n");
        return text;
    }

    /// <summary>
    /// Creates TreeNode for this item to be displayed in Document Explorer TreeView control.
    /// </summary>
    public DefaultMutableTreeNode getTreeNode() throws Exception
	{
		if (mTreeNode == null)
		{
			mTreeNode = new DefaultMutableTreeNode(this);
			if (!mIconNames.contains(getIconName()))
			{
					mIconNames.add(getIconName());
			}
			//java-changed: Work with imagelist was removed
			if (mNode instanceof CompositeNode && ((CompositeNode)mNode).getChildNodes().getCount() > 0)
			{
				mTreeNode.add(new DefaultMutableTreeNode("#dummy"));
			}
		}
		return mTreeNode;
	}

    /// <summary>
    /// Icon to display in the Document Explorer TreeView control.
    /// </summary>
    public ImageIcon getIcon() throws Exception
	{
		if (mIcon == null)
		{
			mIcon = loadIcon(getIconName());
			if (mIcon == null)
				mIcon = loadIcon("Node");
		}
		return mIcon;
	}

    /// <summary>
    /// Icon for this node can be customized by overriding this property in the inheriting classes.
    /// The name represents name of .ico file without extension located in the Icons folder of the project.
    /// </summary>
    protected String getIconName() throws Exception
	{
		return getClass().getSimpleName().replace("Item", "");
	}

    /// <summary>
    /// Provides lazy on-expand loading of underlying tree nodes.
    /// </summary>
    public void onExpand() throws Exception// throws Exception
	{
		if ("#dummy".equals(getTreeNode().getFirstChild().toString()))
		{
			getTreeNode().removeAllChildren();
			Globals.mTreeModel.reload(getTreeNode());
			for (Object o : ((CompositeNode)mNode).getChildNodes())
			{
				Node n = (Node)o;
				getTreeNode().add(Item.createItem(n).getTreeNode());
			}
		}
	}

    /// <summary>
    /// Loads icon from assembly resource stream.
    /// </summary>
    /// <param name="iconName">Name of the icon to load.</param>
    /// <returns>Icon object.</returns>
    private ImageIcon loadIcon(String iconName)
    {
        java.net.URL imgURL = MainForm.class.getResource("images/" + iconName + ".gif");
        if(imgURL != null)
                return new ImageIcon(imgURL);
        else
                return null;
    }

	public void remove() throws Exception
	{
		if (this.isRemovable())
		{
            mNode.remove();
            TreeNode parent = mTreeNode.getParent();
            mTreeNode.removeFromParent();
            Globals.mTreeModel.reload(parent);
            TreePath path = new TreePath(Globals.mRootNode);
            Globals.mTree.setSelectionPath(path);
        }
	}

    public boolean isRemovable()
	{
		return true;
    }

    /// <summary>
    /// Static ctor.
    /// </summary>
    static
    {
		//Fill set of typenames of Item inheritors for Item class fabric.
		mItemSet = new HashMap<String, String>();
		for(Class clazz : DocumentItems.class.getDeclaredClasses())
		{
			try
			{
				String nodeTypeString = (String) clazz.getField("NODE_TYPE_STRING").get(null);
				mItemSet.put(nodeTypeString, clazz.getName());
			}
			catch (Exception e)
			{
				//IllegalAccessException, NoSuchFieldException or NoSuchMethodException - skip such a classes if any.
			}
		}

		//Fill control chars fields set
		mControlCharFields = new ArrayList<java.lang.reflect.Field>();
		Field[] fields = ControlChar.class.getFields();
		for(Field fieldInfo : fields)
		{
			if(fieldInfo.getType() == char.class && Modifier.isStatic(fieldInfo.getModifiers()))
			{
				mControlCharFields.add(fieldInfo);
			}
		}

		//java-added: fills the map of node type integer vaulues to names
		mNodeTypes = new HashMap<Integer, String>();
		Field[] nodeTypefields = NodeType.class.getFields();
		for(Field fieldInfo : nodeTypefields)
		{
			if (fieldInfo.getType() == int.class && Modifier.isStatic(fieldInfo.getModifiers()))
			{
				try
				{
					int integerValue = fieldInfo.getInt(null);
					mNodeTypes.put(integerValue, fieldInfo.getName());
				}
				catch (IllegalAccessException e)
				{
					//skip such a fields if any
				}
			}
		}

		//java-added: fills the map of header/footer type integer vaulues to names
		mHeaderFooterTypes = new HashMap<Integer, String>();
		fields = HeaderFooterType.class.getFields();
		for(Field fieldInfo : fields)
		{
			if(fieldInfo.getType() == int.class && Modifier.isStatic(fieldInfo.getModifiers()))
			{
				try
				{
					int integerValue = fieldInfo.getInt(null);
					mHeaderFooterTypes.put(integerValue, fieldInfo.getName());
				}
				catch (IllegalAccessException e)
				{
					//skip such a fields if any
				}
			}
		}
    }

    /// <summary>
    /// Item class factory implementation.
    /// </summary>
    public static Item createItem(Node node) throws ClassNotFoundException, NoSuchMethodException,
													IllegalAccessException, InvocationTargetException,
													InstantiationException
	{
		String typeName = getNodeTypeString(node);
		if (mItemSet.containsKey(typeName))
			return (Item)Class.forName(mItemSet.get(typeName)).
					getConstructor(DocumentItems.class, Node.class).
					newInstance(null, node);
		else
			return new Item(node);
	}

    //java-added: Object.toString method used by Tree 
    public String toString()
    {
		//java: introduced non-checked RuntimeException on purpose to not change Object.toString() signature
		try
		{
			return getName();
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
	}

    //java-added: convert numerical representaion of the Node type to the string
    private static String getNodeTypeString(Node node)
    {
        int nodeType = node.getNodeType();
        if(mNodeTypes.containsKey(nodeType))
            return mNodeTypes.get(nodeType);
        else
            return "";
    }

    //java-added: convert numerical representaion of the HeaderFooter type to the string
    protected static String getHeaderFooterTypeAsString(HeaderFooter headerFooter) throws Exception
    {
		int headerFooterType = headerFooter.getHeaderFooterType();
		if(mHeaderFooterTypes.containsKey(headerFooterType))
			return mHeaderFooterTypes.get(headerFooterType);
		else
			return "";
    }
}